unit CPU_Usage;

interface

uses
   Wintypes, WinProcs, SysUtils,
   Dialogs, Classes, ExtCtrls, VersionCheck;

const
   SystemProcessorPerformanceInformation = $08;
const
   SystemProcessorPerformanceInformationEx = $8d;

const
   MAXCPUS = 256;

type
   NTSTATUS = longword;

type
   _SYSTEM_INFORMATION_CLASS = DWORD;
   SYSTEM_INFORMATION_CLASS  = _SYSTEM_INFORMATION_CLASS;

type
   SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX = record
      IdleTime:       LARGE_INTEGER;
      KernelTime:     LARGE_INTEGER;
      UserTime:       LARGE_INTEGER;
      DpcTime:        LARGE_INTEGER;
      InterruptTime:  LARGE_INTEGER;
      InterruptCount: ULONG;
      Spare0: ULONG;
      AvailableTime: LARGE_INTEGER;
      Spare1: LARGE_INTEGER;
      Spare2: LARGE_INTEGER;
   end;
   PSYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX = ^SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX;

type
   SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION = record
      IdleTime:       LARGE_INTEGER;
      KernelTime:     LARGE_INTEGER;
      UserTime:       LARGE_INTEGER;
      DpcTime:        LARGE_INTEGER;
      InterruptTime:  LARGE_INTEGER;
      InterruptCount: ULONG;
   end;
   PSYSTEM_PROCESSOR_PERFORMANCE_INFORMATION = ^SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION;

type
   PPerfObj = ^TPerfObj;

   TPerfObj = record
      StartTime: LARGE_INTEGER;
      EndTime:   LARGE_INTEGER;
      StartInfo: array[0..MAXCPUS] of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION;
      EndInfo:   array[0..MAXCPUS] of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION;
   end ;

type
   PPerfObj_ex = ^TPerfObj_ex;

   TPerfObj_ex = record
      StartTime: LARGE_INTEGER;
      EndTime:   LARGE_INTEGER;
      StartInfo: array[0..MAXCPUS] of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX;
      EndInfo:   array[0..MAXCPUS] of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX;
   end;



function NtQuerySystemInformation(SystemInformationClass: SYSTEM_INFORMATION_CLASS;
   SystemInformation: Pointer; SystemInformationLength: ULONG;
   ReturnLength: PULONG): NTSTATUS;
   stdcall; {$IFNDEF DYNAMIC_LINK} external 'ntdll.dll' Name 'NtQuerySystemInformation';
 {$ENDIF}


var
   TickHz:    LARGE_INTEGER;
   PerfCntrs: PPerfObj;
   PerfCntrs_ex: PPerfObj_ex;
   CPUCount_: DWORD = 0;
   OSVer : Integer;

function InitPerfCntrs(): PPerfObj;
function InitPerfCntrs_ex(): PPerfObj_ex;
procedure cpu_util_init();
procedure RestartPerfCntrs(PerfCntrs: PPerfObj);
procedure RestartPerfCntrs_ex(PerfCntrs_ex: PPerfObj_ex);

type
   TCPUCoreUsage = record
      UsageCore: double;
   end;

type
   TAllCPUCoreUsage = array of TCPUCoreUsage;

type
   TCPUUsage = record
      Cores: TAllCPUCoreUsage;
      Total: double;
   end;



type
   TCPUUsageEvent = procedure(CPUUsage: TCPUUsage) of object;

   TCPUReport = class(TComponent)
   private
      FTimer:         TTimer;
      FCPUUsage:      TCPUUsage;
      FCPUCount:      DWORD;
      FCPUUsageEvent: TCPUUsageEvent;
      Fint:           integer;
      procedure FTimerOnTimer(Sender: TObject);
      function ReportPerfCntrs(PerfCntrs: PPerfObj): double;
      function ReportPerfCntrsEx(PerfCntrs_ex: PPerfObj_ex): double;
      function GetCPUCount: DWORD;
   public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;

   published
      property CPUCount: DWORD Read GetCPUCount;
      property onReport: TCPUUsageEvent Read FCPUUsageEvent Write FCPUUsageEvent;
   end;

implementation


procedure cpu_util_init();
begin
   if (@NtQuerySystemInformation <> nil) then
   begin
   if OSVer < 14  then
      PerfCntrs := InitPerfCntrs()
   else
      PerfCntrs_ex := InitPerfCntrs_ex();
   end;

end;

function ReadPerformanceCounter(): LARGE_INTEGER; inline;
var
   Counter: LARGE_INTEGER;
begin
   QueryPerformanceCounter(Counter.QuadPart);
   Result := Counter;
end;



procedure RestartPerfCntrs(PerfCntrs: PPerfObj);
var
   returnLength: DWORD;
   returnNumCPUs: DWORD;
   I: integer;

   status: DWORD;
   SystemInfo: TSystemInfo;

begin
   returnLength := 0;
   GetSystemInfo(SystemInfo);

   CopyMemory(@PerfCntrs^.StartInfo[0], @PerfCntrs^.EndInfo[0], sizeof(
      SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION) * (MAXCPUS + 1));


   PerfCntrs^.StartTime := PerfCntrs^.EndTime;

   status := NtQuerySystemInformation(SystemProcessorPerformanceInformation,
      @PerfCntrs^.EndInfo[0], sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION) *
      MAXCPUS, @returnLength);


   if (status <> 0) then
   begin

      ShowMessage('NtQuery failed, status: ' + IntToStr(status));
      exit;
   end;

   PerfCntrs^.EndTime := ReadPerformanceCounter();


   if ((returnLength mod sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION)) <> 0) then
   begin

      ShowMessage('NtQuery didn''t return expected amount of data');
      Exit;
   end;

   returnNumCPUs := returnLength div sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION);
   CPUCount_ := returnNumCPUs;

   ZeroMemory(@PerfCntrs^.EndInfo[returnNumCPUs],
      sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION) * (MAXCPUS + 1 - returnNumCPUs));


   for I := 0 to returnNumCPUs - 1 do
   begin
      Dec(PerfCntrs^.EndInfo[i].KernelTime.QuadPart, PerfCntrs^.EndInfo[i].IdleTime.QuadPart);
      Inc(PerfCntrs^.EndInfo[MAXCPUS].IdleTime.QuadPart,
         PerfCntrs^.EndInfo[i].IdleTime.QuadPart);
      Inc(PerfCntrs^.EndInfo[MAXCPUS].KernelTime.QuadPart,
         PerfCntrs^.EndInfo[i].KernelTime.QuadPart);
      Inc(PerfCntrs^.EndInfo[MAXCPUS].UserTime.QuadPart,
         PerfCntrs^.EndInfo[i].UserTime.QuadPart);
      Inc(PerfCntrs^.EndInfo[MAXCPUS].DpcTime.QuadPart,
         PerfCntrs^.EndInfo[i].DpcTime.QuadPart);
      Inc(PerfCntrs^.EndInfo[MAXCPUS].InterruptTime.QuadPart,
         PerfCntrs^.EndInfo[i].InterruptTime.QuadPart);
      Inc(PerfCntrs^.EndInfo[MAXCPUS].InterruptCount,
         PerfCntrs^.EndInfo[i].InterruptCount);

   end;

end;

procedure RestartPerfCntrs_ex(PerfCntrs_ex: PPerfObj_ex);
var
   returnLength: DWORD;
   returnNumCPUs: DWORD;
   I: integer;

   status: DWORD;
   SystemInfo: TSystemInfo;

begin
   returnLength := 0;
   GetSystemInfo(SystemInfo);

   CopyMemory(@PerfCntrs_ex^.StartInfo[0], @PerfCntrs_ex^.EndInfo[0], sizeof(
      SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX) * (MAXCPUS + 1));


   PerfCntrs_ex^.StartTime := PerfCntrs_ex^.EndTime;

   status := NtQuerySystemInformation(SystemProcessorPerformanceInformationEx,
      @PerfCntrs_ex^.EndInfo[0], sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX) *
      MAXCPUS, @returnLength);


   if (status <> 0) then
   begin

      ShowMessage('NtQuery failed, status: ' + IntToStr(status));
      exit;
   end;

   PerfCntrs_ex^.EndTime := ReadPerformanceCounter();


   if ((returnLength mod sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX)) <> 0) then
   begin

      ShowMessage('NtQuery didn''t return expected amount of data');
      Exit;
   end;

   returnNumCPUs := returnLength div sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX);
   CPUCount_ := returnNumCPUs;

   ZeroMemory(@PerfCntrs_ex^.EndInfo[returnNumCPUs],
      sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX) * (MAXCPUS + 1 - returnNumCPUs));


   for I := 0 to returnNumCPUs - 1 do
   begin
      Dec(PerfCntrs_ex^.EndInfo[i].KernelTime.QuadPart,   PerfCntrs_ex^.EndInfo[i].IdleTime.QuadPart);
      Inc(PerfCntrs_ex^.EndInfo[MAXCPUS].IdleTime.QuadPart,  PerfCntrs_ex^.EndInfo[i].IdleTime.QuadPart);
      Inc(PerfCntrs_ex^.EndInfo[MAXCPUS].KernelTime.QuadPart,   PerfCntrs_ex^.EndInfo[i].KernelTime.QuadPart);
      Inc(PerfCntrs_ex^.EndInfo[MAXCPUS].UserTime.QuadPart,   PerfCntrs_ex^.EndInfo[i].UserTime.QuadPart);
      Inc(PerfCntrs_ex^.EndInfo[MAXCPUS].DpcTime.QuadPart,   PerfCntrs_ex^.EndInfo[i].DpcTime.QuadPart);
      Inc(PerfCntrs_ex^.EndInfo[MAXCPUS].InterruptTime.QuadPart,  PerfCntrs_ex^.EndInfo[i].InterruptTime.QuadPart);
      Inc(PerfCntrs_ex^.EndInfo[MAXCPUS].InterruptCount,   PerfCntrs_ex^.EndInfo[i].InterruptCount);

   end;

end;

function TCPUReport.ReportPerfCntrs(PerfCntrs: PPerfObj): double;
var
   tot_CPU_Util: double;
   I: integer;
   duration: double;
   ActualDuration: LARGE_INTEGER;
   DeltaInfo: array[0..MAXCPUS] of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION;
   TotalCPUTime: array[0..MAXCPUS] of LARGE_INTEGER;
   SystemInfo: TSystemInfo;
begin

   GetSystemInfo(SystemInfo);

   for I := 0 to MAXCPUS do
   begin
      DeltaInfo[I].IdleTime.QuadPart :=
         PerfCntrs^.EndInfo[i].IdleTime.QuadPart - PerfCntrs^.StartInfo[i].IdleTime.QuadPart;

      DeltaInfo[i].KernelTime.QuadPart :=
         PerfCntrs^.EndInfo[i].KernelTime.QuadPart - PerfCntrs^.StartInfo[i].KernelTime.QuadPart;
      DeltaInfo[i].UserTime.QuadPart :=
         PerfCntrs^.EndInfo[i].UserTime.QuadPart - PerfCntrs^.StartInfo[i].UserTime.QuadPart;
      DeltaInfo[i].DpcTime.QuadPart :=
         PerfCntrs^.EndInfo[i].DpcTime.QuadPart -
         PerfCntrs^.StartInfo[i].DpcTime.QuadPart;
      DeltaInfo[i].InterruptTime.QuadPart :=
         PerfCntrs^.EndInfo[i].InterruptTime.QuadPart -
         PerfCntrs^.StartInfo[i].InterruptTime.QuadPart;
      DeltaInfo[i].InterruptCount :=
         PerfCntrs^.EndInfo[i].InterruptCount - PerfCntrs^.StartInfo[i].InterruptCount;

      TotalCPUTime[i].QuadPart :=
         DeltaInfo[i].IdleTime.QuadPart + DeltaInfo[i].KernelTime.QuadPart +
         DeltaInfo[i].UserTime.QuadPart;
   end;

   tot_CPU_Util := 100.0 * (1.0 - DeltaInfo[MAXCPUS].IdleTime.QuadPart / TotalCPUTime[MAXCPUS].QuadPart);
   //ShowMessage(Format('%5.2f', [tot_CPU_Util]));

   ActualDuration.QuadPart := PerfCntrs^.EndTime.QuadPart - PerfCntrs^.StartTime.QuadPart;


   ActualDuration.QuadPart := Trunc((ActualDuration.QuadPart * 10000) / TickHz.QuadPart);
   duration := ActualDuration.QuadPart / 10.0;

   with FCPUUsage do
   begin
     Total := tot_CPU_Util;
     for I := Low(Cores) to High(Cores) do
     begin
        Cores[I].UsageCore := 100.0 * (1.0 -  DeltaInfo[i].IdleTime.QuadPart /  TotalCPUTime[i].QuadPart);
     end;
   end;

   if Assigned(FCPUUsageEvent) then
   begin
       onReport(FCPUUsage);
   end;
   Result := tot_CPU_Util;
end;

function TCPUReport.ReportPerfCntrsEx(PerfCntrs_ex: PPerfObj_ex): double;
var
   tot_CPU_Util: double;
   I: integer;
   duration: double;
   ActualDuration: LARGE_INTEGER;
   DeltaInfo: array[0..MAXCPUS] of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION_EX;
   TotalCPUTime: array[0..MAXCPUS] of LARGE_INTEGER;
   SystemInfo: TSystemInfo;
begin

   GetSystemInfo(SystemInfo);

   for I := 0 to MAXCPUS do
   begin
      DeltaInfo[I].IdleTime.QuadPart := PerfCntrs_ex^.EndInfo[i].IdleTime.QuadPart - PerfCntrs_ex^.StartInfo[i].IdleTime.QuadPart;

      DeltaInfo[i].KernelTime.QuadPart := PerfCntrs_ex^.EndInfo[i].KernelTime.QuadPart - PerfCntrs_ex^.StartInfo[i].KernelTime.QuadPart;

      DeltaInfo[i].UserTime.QuadPart := PerfCntrs_ex^.EndInfo[i].UserTime.QuadPart - PerfCntrs_ex^.StartInfo[i].UserTime.QuadPart;
      DeltaInfo[i].DpcTime.QuadPart := PerfCntrs_ex^.EndInfo[i].DpcTime.QuadPart - PerfCntrs_ex^.StartInfo[i].DpcTime.QuadPart;
      DeltaInfo[i].InterruptTime.QuadPart := PerfCntrs_ex^.EndInfo[i].InterruptTime.QuadPart - PerfCntrs_ex^.StartInfo[i].InterruptTime.QuadPart;
      DeltaInfo[i].InterruptCount := PerfCntrs_ex^.EndInfo[i].InterruptCount - PerfCntrs_ex^.StartInfo[i].InterruptCount;

      TotalCPUTime[i].QuadPart :=
         DeltaInfo[i].IdleTime.QuadPart + DeltaInfo[i].KernelTime.QuadPart +
         DeltaInfo[i].UserTime.QuadPart;
   end;

   tot_CPU_Util := 100.0 * (1.0 - DeltaInfo[MAXCPUS].IdleTime.QuadPart / TotalCPUTime[MAXCPUS].QuadPart);
   //ShowMessage(Format('%5.2f', [tot_CPU_Util]));

   ActualDuration.QuadPart := PerfCntrs_ex^.EndTime.QuadPart - PerfCntrs_ex^.StartTime.QuadPart;


   ActualDuration.QuadPart := Trunc((ActualDuration.QuadPart * 10000) / TickHz.QuadPart);
   duration := ActualDuration.QuadPart / 10.0;

   with FCPUUsage do
   begin
     Total := tot_CPU_Util;
     for I := Low(Cores) to High(Cores) do
     begin
        Cores[I].UsageCore := 100.0 * (1.0 -  DeltaInfo[i].IdleTime.QuadPart /  TotalCPUTime[i].QuadPart);
     end;
   end;

   if Assigned(FCPUUsageEvent) then
   begin
       onReport(FCPUUsage);
   end;
   Result := tot_CPU_Util;

end;

function InitPerfCntrs(): PPerfObj;
var
   NewPerfCntrs: PPerfObj;
   NTVersion: DWORD;
   status: DWORD;
   SystemInfo: TSystemInfo;
begin
   GetSystemInfo(SystemInfo);
   NewPerfCntrs := PPerfObj(GlobalAlloc(GPTR, sizeof(TPerfObj)));
   assert(NewPerfCntrs <> nil);
   ZeroMemory(PChar(NewPerfCntrs), sizeof(TPerfObj));

   NTVersion := GetVersion();
   if (NTVersion >= $80000000) then
   begin
      ShowMessage('Not running on Windows NT');
      Exit;
   end;

   if (QueryPerformanceFrequency(TickHz.QuadPart) = False) then
   begin
      ShowMessage('QueryPerformanceFrequency Failed!');
      Exit;
   end;

   RestartPerfCntrs(NewPerfCntrs);

   Result := NewPerfCntrs;

end;

function InitPerfCntrs_ex(): PPerfObj_ex;
var
   NewPerfCntrs_ex: PPerfObj_ex;
   NTVersion: DWORD;
   status: DWORD;
   SystemInfo: TSystemInfo;
begin
   GetSystemInfo(SystemInfo);
   NewPerfCntrs_ex := PPerfObj_ex(GlobalAlloc(GPTR, sizeof(TPerfObj_ex)));
   assert(NewPerfCntrs_ex <> nil);
   ZeroMemory(PChar(NewPerfCntrs_ex), sizeof(TPerfObj_ex));

   NTVersion := GetVersion();
   if (NTVersion >= $80000000) then
   begin
      ShowMessage('Not running on Windows NT');
      Exit;
   end;

   if (QueryPerformanceFrequency(TickHz.QuadPart) = False) then
   begin
      ShowMessage('QueryPerformanceFrequency Failed!');
      Exit;
   end;

   RestartPerfCntrs_ex(NewPerfCntrs_ex);

   Result := NewPerfCntrs_ex;

end;

{ TCPUUsage }

constructor TCPUReport.Create(AOwner: TComponent);
begin
   inherited;
   Fint := 0;
   FCPUCount := CPUCount;
   SetLength(FCPUUsage.Cores, CPUCount);
   FTimer := TTimer.Create(Self);
   FTimer.Interval := 1000;
   FTimer.OnTimer := FTimerOnTimer;
   FTimer.Enabled := True;
end;

destructor TCPUReport.Destroy;
begin
   FTimer.Enabled := False;
   FTimer.Free;
   inherited;
end;

procedure TCPUReport.FTimerOnTimer(Sender: TObject);
begin
   if OSVer < 14 then
   begin
      RestartPerfCntrs(PerfCntrs);
      if Fint = 0 then
         Fint := 1
      else
         ReportPerfCntrs(PerfCntrs);
      end
   else
   begin
      RestartPerfCntrs_ex(PerfCntrs_ex);
      if Fint = 0 then
         Fint := 1
      else
         ReportPerfCntrsEx(PerfCntrs_ex);
   end
end;

function TCPUReport.GetCPUCount: DWORD;
begin
   Result := CPUCount_;
end;

initialization
   TickHz.LowPart  := 0;
   TickHz.HighPart := 0;
   OSVer := Integer(WinVersion);
   cpu_util_init();

   //ShowMessage(IntToStr(OSVer));

end.

