unit HDD_Usage;

interface

uses
   Classes, ExtCtrls, Windows, Dialogs, PDH;

type
   THDDUsageEvent = procedure(Usage: double) of object;

   THDDReport = class(TComponent)
   private
      FCanPDH:        boolean;
      FhddQueryTime:  HQUERY;
      FhddTotalTime:  HCOUNTER;
      FhddQueryIdle:  HQUERY;
      FhddTotalIdle:  HCOUNTER;
      FHDDUsageEvent: THDDUsageEvent;
      FTimer:         TTimer;
      function init: boolean;
      procedure FTimerOnTimer(Sender: TObject);
      function getHDDTimeValue: double;
      function getHDDIdleValue: double;
   public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
   published
      property onReport: THDDUsageEvent Read FHDDUsageEvent Write FHDDUsageEvent;
   end;




implementation

{ THDDReport }

function THDDReport.init: boolean;
var
   dwResult: longint;
begin
   Result := False;
   dwResult := PdhOpenQuery(nil, 0, FhddQueryTime);
   if dwResult = 0 then
   begin
      if PdhAddCounter(FhddQueryTime, '\PhysicalDisk(_Total)\% Disk Time', 0,
         FhddTotalTime) = ERROR_SUCCESS then
      begin
         dwResult := PdhCollectQueryData(FhddQueryTime);
         if dwResult = 0 then
         begin

         end
         else
         begin
            ShowMessage('PdhCollectQueryData error!');
            Exit;
         end;

      end
      else
      begin
         ShowMessage('PdhAddCounter error!');
         Exit;
      end;
   end
   else
   begin
      ShowMessage('PdhOpenQuery error!');
      Exit;
   end;

   dwResult := PdhOpenQuery(nil, 0, FhddQueryIdle);
   if dwResult = 0 then
   begin
      if PdhAddCounter(FhddQueryIdle, '\PhysicalDisk(_Total)\% Idle Time', 0,
         FhddTotalIdle) = ERROR_SUCCESS then
      begin
         dwResult := PdhCollectQueryData(FhddQueryIdle);
         if dwResult = 0 then
         begin
            Result := True;
         end
         else
         begin
            ShowMessage('PdhCollectQueryData error!');
            Exit;
         end;
      end
      else
      begin
         ShowMessage('PdhAddCounter error!');
         Exit;
      end;
   end
   else
   begin
      ShowMessage('PdhOpenQuery error!');
      Exit;
   end;
   Result := True;
end;

constructor THDDReport.Create(AOwner: TComponent);
begin
   inherited;
   FTimer  := TTimer.Create(Self);
   FTimer.Interval := 1000;
   FTimer.OnTimer := FTimerOnTimer;
   FTimer.Enabled := True;
   FCanPDH := init;
end;

destructor THDDReport.Destroy;
begin

   inherited;
end;

procedure THDDReport.FTimerOnTimer(Sender: TObject);
var
   DiskTime, DiskIdle, Usage: double;
begin
   if FCanPDH then
   begin

         DiskIdle := getHDDIdleValue;
         DiskTime := getHDDTimeValue;

         Usage := DiskTime - ((DiskTime + DiskIdle) - 100);

         if Usage < 0 then
            Usage := 0;
         if Usage > 100 then
            Usage := 100;
         if Assigned(FHDDUsageEvent) then
             onReport(Usage);
   end;
end;

function THDDReport.getHDDIdleValue: double;
var
   counterVal: TPdhFmtCounterValue;
begin
   PdhCollectQueryData(FhddQueryIdle);
   PdhGetFormattedCounterValue(FhddTotalIdle, PDH_FMT_DOUBLE, nil, counterVal);
   Result := counterVal.doubleValue;
end;

function THDDReport.getHDDTimeValue: double;
var
   counterVal: TPdhFmtCounterValue;
begin
   PdhCollectQueryData(FhddQueryTime);
   PdhGetFormattedCounterValue(FhddTotalTime, PDH_FMT_DOUBLE, nil, counterVal);
   Result := counterVal.doubleValue;
end;

end.

